<?php

namespace App\Libraries;

use App\Models\User;
use Illuminate\Support\Facades\Crypt;
class ForgotPasswordLink extends LoginOtpEmail {
    public $redirect_url = null;
    public $encryption = FALSE;

    public function decodeLink($code = "", $key = null){

        // Using user generated key...
        // if (!is_null($key)) {
        //     if (strlen($key) < 8) return json_encode(["msg" => "Invalid Key"]);
        //     $this->config->key = $key;
        // }

        if ($this->encryption == TRUE) {
            try {
                $code = Crypt::decrypt($code);
            } catch (\Throwable $th) {
                return json_encode(["msg" => "Failed to decode Link"]);
            }
        } else {
            $code = base64_decode($code);
        }
        $user_id = explode("_", $code)[0];

        // Checking if code belongs to the user...
        $result = User::checkIfCodeBelongsToUser($user_id, $code);
        // Checking if code belongs to the user...

        return $result ? json_encode(["msg" => "Link Validation Successful", "data" => User::fetchUserById($user_id)]) : json_encode(["msg" => "Link Validation Failed"]);
    }


    protected function generateCode($user_id = null, $key = null){
        // Using user generated key...
        // if (!is_null($key)) {
        //     if (strlen($key) < 8) return -1;
        //     $this->config->key = $key;
        // }

        // Checking if some unused code is already active..
        $code = User::checkUnusedCode($user_id);
        // Checking if some unused code is already active..

        if (empty($code)) $code = $user_id . "_" . strtotime(date("Y-m-d H:i:s"));
        return $this->encryption == FALSE ? base64_encode($code) : Crypt::encrypt($code);
    }


    public function generatedLink($email = "", $key = null){
        $user = User::fetchUserDetails($email, "email");
        if (empty($user)) return json_encode(["msg" => "User does not exists"]);

        if (is_null($this->redirect_url)) return json_encode(["msg" => "Invalid Redirect URL"]);

        $code = $this->generateCode($user->id);
        if ($code == -1) return json_encode(["msg" => "Invalid Key"]);

        return json_encode(["msg" => "Link Creation Successful", "link" => $this->redirect_url . '?code=' . $code]);
    }

    public function sendPasswordLink($from = null, $to = null, $subject = null, $msg = null, $link = ""){
        // determine if the link contains the code paramenter...
        if (strpos($link, "?code=") < 0) return json_encode(["msg" => "Invalid Link"]);

        // Send the link via Email...
        $result = json_decode($this->sendEmail($from, $to, $subject, $msg));
        
        if (isset($result->data) && !empty($result->data)) {
            $code = "";
            try {
                $code = $this->encryption == TRUE ? Crypt::decrypt(explode("?code=", $link)[1]) : base64_decode(explode("?code=", $link)[1]);
            } catch (\Throwable $th) {
                return json_encode(["msg" => "Failed to decode Link"]);
            }
            $user_id = explode("_", $code)[0];

            $insert = User::checkUnusedCode($user_id);
            if (!$insert) $insert = User::logSentCode($user_id, $code);

            if (!$insert) return json_encode(["msg" => "Failed to log link sent"]);
        }

        return json_encode($result);
    }

    public function resetPassword($email, $new_password = "", $re_type_password = ""){

        $res = emailValidator($email);
        if (!empty($res)) return $res;

        // Checking if password doesn't match
        $new_password = trim($new_password);
        $re_type_password = trim($re_type_password);
        if ($new_password != $re_type_password) {
            return json_encode(["msg" => "Password doesn't Match"]);
        }
        // Checking if password doesn't match

        $result = User::updateUserPassword($email, $new_password, "email");
        if ($result) {
            $code_updated = User::updateCodeUsed($email);
            if (!$code_updated) return json_encode(["msg" => "Failed to log code used"]);
            return json_encode(["msg" => "Password Updated Successfully"]);
        }

        return json_encode(["msg" => "Failed to update Password"]);
    }
}

?>