<?php

namespace App\Models;

// use Illuminate\Contracts\Auth\MustVerifyEmail;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
class User extends Authenticatable
{
    use HasFactory, Notifiable;

    /**
     * The attributes that are mass assignable.
     *
     * @var array<int, string>
     */
    protected $fillable = [
        'name',
        'email',
        'password',
    ];

    /**
     * The attributes that should be hidden for serialization.
     *
     * @var array<int, string>
     */
    protected $hidden = [
        'password',
        'remember_token',
    ];

    /**
     * Get the attributes that should be cast.
     *
     * @return array<string, string>
     */
    protected function casts(): array
    {
        return [
            'email_verified_at' => 'datetime',
            'password' => 'hashed',
        ];
    }

    static public function fetchUserDetails($param = null,$type = "email"){
        try {
            return DB::table("users")->select("*")->where($type, $param)->get(["id", 'username'])->first();
        } catch (\Throwable $th) {
        } 
        return null;
    }

    static public function logOtp($param = null, $otp = null, $type = null): int
    {
        if ($type != 'mobile_no' && $type != 'email') return 0;
        $data = [
            $type => $param,
            'otp' => $otp
        ];
        try {
            return DB::table("otp")->insert($data);
        } catch (\Throwable $th) {
            return 0;
        }
    }

    static public function checkLastOTP($param = null, $type = "mobile_no")
    {
        $query = "SELECT $type,otp FROM otp where $type = '$param' and id = (SELECT MAX(id) as max_id FROM otp where $type = '$param')";

        try {
            $result = DB::select($query);
            return !empty($result) ? $result[0] : null;
        } catch (\Throwable $th) {
            
        }

        return null;
    }

    static public function checkIfUserExists($param = null, $password = null, $type = "email")
    {
        if ($type != 'email' && $type != 'name') {
            return json_encode(["msg" => "Invalid type validation"]);
        }
        // checking Validation Rules
        if ($type == "email") {
            $email_valid = emailValidator($param);
            if ($email_valid) return $email_valid;
        } else if (empty($param)) {
            return json_encode(["msg" => "Missing Username"]);
        }

        $pass_valid = passwordValidator($password);
        if ($pass_valid) return $pass_valid;
        // checking Validation Rules

        try {
            $user = DB::table("users")->where($type, $param)->get()->first();
        } catch (\Throwable $th) {
            return json_encode(["msg" => "Database / Server Issue"]);
        }

        if (empty($user)) return json_encode(["msg" => "Details Not Found"]);

        return $user;
    }

    static public function updateUserPassword($param = null, $new_password = null, $type = "email"){
        try {
            return DB::update("update users set password = ? where $type = ?",[Hash::make($new_password),$param]);
        } catch (\Throwable $th) {
            return -1;
        }
        return 0;
    }
    static public function checkUnusedCode($user_id){
        try {
           $result = DB::table("password_link")->select("code")->where("code_used",0)->where("user_id",$user_id)->get()->first();
        } catch (\Throwable $th) {
            //throw $th;
        }
        return !empty($result) ? $result->code : null;
    }

    static public  function logSentCode($user_id = "", $code = ""){
        $insertion = 0;
        try {
            $to_insert = [
                "user_id" => $user_id,
                "code" => $code
            ];
            $insertion =  DB::table("password_link")->insert($to_insert);
        } catch (\Throwable $th) {
            $insertion = 0;
            //throw $th;
        }
        return $insertion;
    }  
    static public  function updateCodeUsed($email = ""){
        $updated = 0;
        $user = self::fetchUserDetails($email,"email");
        try {
            $query = "SELECT MAX(id) as id FROM password_link where code_used = 0 and user_id = $user->id";
            
            $max_id = DB::select($query);
            $max_id = !empty($max_id) ? $max_id[0]->id : 0;
            if($max_id){
                return DB::update("update password_link set code_used = ?,used_at = ? where id = ?", [1,current_timestamp(),$max_id]);
            }
        } catch (\Throwable $th) {
            //throw $th;
        }

        return $updated;
    }

    static public function fetchUserById($user_id = null){
        $user = null;
        
        try {
            $user = DB::table("users")->where("id",$user_id)->get()->first();
        } catch (\Throwable $th) {
            throw $th;
        }

        return $user;
    }

    static public function checkIfCodeBelongsToUser($user_id,$code){
        $result = 0;
        try {
            $result = DB::table("password_link")->where("user_id",$user_id)->where("code",$code)->where("code_used",0)->get()->first();
        } catch (\Throwable $th) {
            //throw $th;
        }
        return $result;
    }

    static public function getSessionIdFromdDB(){
        try {
            $result = DB::table("users")->select("session_id")->where("id", session()->get("user_id"))->get()->first();
            return !empty($result) ? $result->session_id : -1;
        } catch (\Throwable $th) {
            //throw $th;
        }

        return -1;
    }
}
